@@ -1,20 +1,4 @@
 from __future__ import print_function
-try:
-    import test.support
-
-    # Skip tests if _multiprocessing wasn't built.
-    test.support.import_module('_multiprocessing')
-    # Skip tests if sem_open implementation is broken.
-    test.support.import_module('multiprocessing.synchronize')
-    # import threading after _multiprocessing to raise a more revelant error
-    # message: "No module named _multiprocessing" if multiprocessing is not
-    # compiled without thread support.
-    test.support.import_module('threading')
-except ImportError:
-    pass
-
-
-# from test.support.script_helper import assert_python_ok
 from loky import process_executor
 
 import os
@@ -22,6 +6,7 @@ import gc
 import sys
 import time
 import shutil
+import platform
 import pytest
 import weakref
 import tempfile
@@ -32,6 +17,7 @@ from math import sqrt
 from threading import Thread
 from collections import defaultdict
 
+import loky
 from loky.process_executor import LokyRecursionError
 from loky.process_executor import ShutdownExecutorError, TerminatedWorkerError
 from loky._base import (PENDING, RUNNING, CANCELLED, CANCELLED_AND_NOTIFIED,
@@ -39,7 +25,8 @@ from loky._base import (PENDING, RUNNING, CANCELLED, CANCELLED_AND_NOTIFIED,
 
 from . import _executor_mixin
 from .utils import id_sleep, check_subprocess_call, filter_match
-from .test_reusable_executor import ErrorAtPickle, ExitAtPickle, c_exit
+from .test_reusable_executor import ErrorAtPickle, ExitAtPickle
+from .test_reusable_executor import PICKLING_ERRORS, c_exit
 
 if sys.version_info[:2] < (3, 3):
     import loky._base as futures
@@ -116,7 +103,7 @@ class ExecutorShutdownTest:
             e.submit(id, ExitAtPickle())
 
     def test_interpreter_shutdown(self):
-        # Free ressources to avoid random timeout in CI
+        # Free resources to avoid random timeout in CI
         self.executor.shutdown(wait=True, kill_workers=True)
 
         tempdir = tempfile.mkdtemp(prefix='loky_')
@@ -297,7 +284,7 @@ class ExecutorShutdownTest:
     def test_del_shutdown(self):
         executor = self.executor_type(max_workers=5, context=self.context)
         list(executor.map(abs, range(-5, 5)))
-        queue_management_thread = executor._queue_management_thread
+        executor_manager_thread = executor._executor_manager_thread
         processes = executor._processes
         del executor
         if IS_PYPY:
@@ -305,10 +292,108 @@ class ExecutorShutdownTest:
             time.sleep(1.)
             gc.collect()
 
-        queue_management_thread.join()
+        executor_manager_thread.join()
         for p in processes.values():
             p.join()
 
+    @classmethod
+    def _wait_and_return(cls, x):
+        # This _test_event is passed globally through an initializer to
+        # the executor.
+        _executor_mixin._test_event.wait()
+        return x
+
+    def test_shutdown_no_wait(self):
+        # Ensure that the executor cleans up the processes when calling
+        # shutdown with wait=False
+
+        # Stores executor internals to be able to check that the executor
+        # shutdown correctly
+        processes = self.executor._processes
+        call_queue = self.executor._call_queue
+        executor_manager_thread = self.executor._executor_manager_thread
+
+        # submit tasks that will finish after the shutdown and make sure they
+        # were started
+        res = [self.executor.submit(self._wait_and_return, x)
+               for x in range(-5, 5)]
+
+        self.executor.shutdown(wait=False)
+
+        with pytest.raises(ShutdownExecutorError):
+            # It's no longer possible to submit any new tasks to this
+            # executor after shutdown.
+            self.executor.submit(lambda x: x, 42)
+
+        # Check that even after shutdown, all futures are still running
+        assert all(f._state in (PENDING, RUNNING) for f in res)
+
+        # Let the futures finish and make sure that all the executor resources
+        # were properly cleaned by the shutdown process
+        _executor_mixin._test_event.set()
+        executor_manager_thread.join()
+        for p in processes.values():
+            p.join()
+        call_queue.join_thread()
+
+        # Make sure the results were all computed before the executor
+        # resources were freed.
+        assert all([f.result() == v for f, v in zip(res, range(-5, 5))])
+
+    def test_shutdown_deadlock_pickle(self):
+        # Test that the pool calling shutdown with wait=False does not cause
+        # a deadlock if a task fails at pickle after the shutdown call.
+        # Reported in bpo-39104.
+        self.executor.shutdown(wait=True)
+        with self.executor_type(max_workers=2,
+                                context=self.context) as executor:
+            self.executor = executor  # Allow clean up in fail_on_deadlock
+
+            # Start the executor and get the executor_manager_thread to collect
+            # the threads and avoid dangling thread that should be cleaned up
+            # asynchronously.
+            executor.submit(id, 42).result()
+            executor_manager = executor._executor_manager_thread
+
+            # Submit a task that fails at pickle and shutdown the executor
+            # without waiting
+            f = executor.submit(id, ErrorAtPickle())
+            executor.shutdown(wait=False)
+            with pytest.raises(PICKLING_ERRORS):
+                f.result()
+
+        # Make sure the executor is eventually shutdown and do not leave
+        # dangling threads
+        executor_manager.join()
+
+    def test_hang_issue39205(self):
+        """shutdown(wait=False) doesn't hang at exit with running futures.
+
+        See https://bugs.python.org/issue39205.
+        """
+        code = """if True:
+            from loky.process_executor import {executor_type}
+            from loky.backend import get_context
+            from tests._test_process_executor import sleep_and_print
+
+            context = get_context("{start_method}")
+            e = {executor_type}(3, context=context)
+
+            e.submit(sleep_and_print, 1.0, "apple")
+            e.shutdown(wait=False)
+        """
+        code = code.format(executor_type=self.executor_type.__name__,
+                           start_method=self.context.get_start_method())
+        stdout, stderr = check_subprocess_call(
+                [sys.executable, "-c", code], timeout=55)
+
+        # On OSX, remove UserWarning for broken semaphores
+        if sys.platform == "darwin":
+            stderr = [e for e in stderr.strip().split("\n")
+                      if "increase its maximal value" not in e]
+        assert len(stderr) == 0 or stderr[0] == ''
+        assert stdout.strip() == "apple"
+
     @classmethod
     def _test_recursive_kill(cls, depth):
         executor = cls.executor_type(
@@ -535,9 +620,19 @@ class ExecutorTest:
         self.executor.map(str, [2] * (self.worker_count + 1))
         self.executor.shutdown()
 
+    @pytest.mark.skipif(
+            platform.python_implementation() != "CPython" or
+            (sys.version_info >= (3, 8, 0) and sys.version_info < (3, 8, 2)),
+            reason="Underlying bug fixed upstream starting Python 3.8.2")
     def test_no_stale_references(self):
         # Issue #16284: check that the executors don't unnecessarily hang onto
         # references.
+
+        # This test has to be skipped on early Python 3.8 versions because of a
+        # low-level reference cycle inside the pickle module for early versions
+        # of Python 3.8 preventing stale references from being collected. See
+        # cloudpipe/cloudpickle#327 as well as
+        # https://bugs.python.org/issue39492
         my_object = MyObject()
         collect = threading.Event()
         _ = weakref.ref(my_object, lambda obj: collect.set())  # noqa
@@ -793,11 +888,23 @@ class ExecutorTest:
                                  ctx=self.context)
 
     @pytest.mark.high_memory
-    @pytest.mark.skipif(sys.version_info < (3, 4),
-                        reason="Limitation only present for python3")
-    def test_failure_on_large_data_send(self):
+    @pytest.mark.skipif(sys.maxsize < 2 ** 32,
+                        reason="Test requires a 64 bit version of Python")
+    @pytest.mark.skipif(
+            sys.version_info[:2] < (3, 8),
+            reason="Python version does not support pickling objects of size > 2 ** 31GB")
+    def test_no_failure_on_large_data_send(self):
         data = b'\x00' * int(2.2e9)
+        self.executor.submit(id, data).result()
 
+    @pytest.mark.high_memory
+    @pytest.mark.skipif(sys.maxsize < 2 ** 32,
+                        reason="Test requires a 64 bit version of Python")
+    @pytest.mark.skipif(
+            sys.version_info[:2] >= (3, 8),
+            reason="Python version supports pickling objects of size > 2 ** 31GB")
+    def test_expected_failure_on_large_data_send(self):
+        data = b'\x00' * int(2.2e9)
         with pytest.raises(RuntimeError):
             self.executor.submit(id, data).result()
 
@@ -806,7 +913,7 @@ class ExecutorTest:
 
         executor = self.executor_type(1, context=self.context)
 
-        def _leak_some_memory(size=int(1e6), delay=0.001):
+        def _leak_some_memory(size=int(3e6), delay=0.001):
             """function that leaks some memory """
             from loky import process_executor
             process_executor._MEMORY_LEAK_CHECK_DELAY = 0.1
@@ -841,7 +948,7 @@ class ExecutorTest:
             # default process size + what has leaked since the last
             # memory check.
             for _, leak_size in results:
-                assert leak_size / 1e6 < 250
+                assert leak_size / 1e6 < 650
 
     def test_reference_cycle_collection(self):
         # make the parallel call create a reference cycle and make
@@ -892,3 +999,95 @@ class ExecutorTest:
         # Submitting other jobs fails as well.
         with pytest.raises(TerminatedWorkerError, match=match):
             self.executor.submit(pow, 2, 8)
+
+    @staticmethod
+    def _test_child_env(var_name):
+        import os
+        return os.environ.get(var_name, "unset")
+
+    def test_child_env_executor(self):
+        # Test that for loky context, setting argument env correctly overwrite
+        # the environment of the child process.
+        if self.context.get_start_method() != 'loky':
+            pytest.skip(msg="Only work with loky context")
+
+        var_name = "loky_child_env_executor"
+        var_value = "variable set"
+        executor = self.executor_type(1, env={var_name: var_value})
+
+        var_child = executor.submit(self._test_child_env, var_name).result()
+        assert var_child == var_value
+
+        executor.shutdown(wait=True)
+
+    def test_viztracer_profiler(self):
+        # Check that viztracer profiler is initialzed in workers when
+        # installed.
+        viztracer = pytest.importorskip("viztracer")
+
+        def check_viztracer_active():
+            tracer = viztracer.get_tracer()
+            if tracer is None:
+                return False
+            return tracer.enable
+
+        active_in_main_process = check_viztracer_active()
+        with self.executor_type(1, context=self.context) as e:
+            active_in_child_process = e.submit(check_viztracer_active).result()
+        assert active_in_main_process == active_in_child_process
+
+        if not active_in_main_process:
+            tracer = viztracer.VizTracer()
+            try:
+                tracer.start()
+                with self.executor_type(1, context=self.context) as e:
+                    assert e.submit(check_viztracer_active).result()
+            finally:
+                tracer.stop()
+
+            # Once the tracer has been stopped, should be no side effect on
+            # workers started in new executors.
+            with self.executor_type(1, context=self.context) as e:
+                assert not e.submit(check_viztracer_active).result()
+
+    def test_viztracer_profiler_with_custom_init(self):
+        # Check that viztracer profiler is initialzed in workers when
+        # installed.
+        viztracer = pytest.importorskip("viztracer")
+
+        # Make sure the auto-viztracer initialization works even when
+        # the call pass their own init.
+
+        def check_viztracer_active_and_custom_init():
+            assert loky._custom_global_var == 42
+            tracer = viztracer.get_tracer()
+            if tracer is None:
+                return False
+            return tracer.enable
+
+        existing_tracer = viztracer.get_tracer()
+        if existing_tracer is not None and existing_tracer.enable:
+            pytest.skip("Cannot run this test if viztracer is active")
+
+        tracer = viztracer.VizTracer()
+        try:
+            tracer.start()
+            with self.executor_type(
+                1,
+                context=self.context,
+                initializer=_custom_initializer
+            ) as e:
+                assert e.submit(
+                    check_viztracer_active_and_custom_init
+                ).result()
+        finally:
+            tracer.stop()
+
+
+def _custom_initializer():
+    """_custom_initializer is module function to be picklable
+
+    This is necessary for executor implementations that do not
+    use cloudpickle to pickle the initializer.
+    """
+    loky._custom_global_var = 42
