@@ -5,6 +5,7 @@ import psutil
 import pytest
 import signal
 import pickle
+import platform
 import socket
 import multiprocessing as mp
 from tempfile import mkstemp
@@ -13,12 +14,9 @@ from loky.backend import get_context
 from loky.backend.compat import wait
 from loky.backend.context import START_METHODS
 from loky.backend.utils import recursive_terminate
-from .utils import TimingWrapper, check_subprocess_call
 
-try:
-    from ._openmp.parallel_sum import parallel_sum
-except ImportError:
-    parallel_sum = None
+from .utils import TimingWrapper, check_subprocess_call
+from .utils import with_parallel_sum, _run_openmp_parallel_sum
 
 if sys.version_info < (3, 3):
     FileNotFoundError = NameError
@@ -278,6 +276,45 @@ class TestLokyBackend:
         parent_connection.close()
         child_connection.close()
 
+    @staticmethod
+    def _test_child_env(key, queue):
+        import os
+
+        queue.put(os.environ.get(key, 'not set'))
+
+    @pytest.mark.xfail(sys.version_info < (3, 6) and sys.platform == "win32",
+                       reason="Can randomly fail with python < 3.6 under windows.")
+    def test_child_env_process(self):
+        import os
+
+        key = 'loky_child_env_process'
+        value = 'loky works'
+        out_queue = self.SimpleQueue()
+        try:
+            # Test that the environment variable is correctly copied in the
+            # child process.
+            os.environ[key] = value
+            p = self.Process(target=self._test_child_env,
+                             args=(key, out_queue))
+            p.start()
+            child_var = out_queue.get()
+            p.join()
+
+            assert child_var == value
+
+            # Test that the environment variable is correctly overwritted by
+            # using the `env` argument in Process.
+            new_value = 'loky rocks'
+            p = self.Process(target=self._test_child_env,
+                             args=(key, out_queue), env={key: new_value})
+            p.start()
+            child_var = out_queue.get()
+            p.join()
+
+            assert child_var == new_value, p.env
+        finally:
+            del os.environ[key]
+
     @classmethod
     def _test_terminate(cls, event):
         # Notify the main process that child process started
@@ -503,10 +540,19 @@ class TestLokyBackend:
             # no other files should be opened at this stage in the process
             assert (is_pipe or is_std or is_rng or is_mem)
 
-        # there should be one pipe for communication with main process
-        # and the semaphore tracker pipe and the Connection pipe
-        assert n_pipe == 3, ("Some pipes were not properly closed during the "
-                             "child process setup.")
+        # there should be:
+        # - one pipe for communication with main process
+        # - loky's resource_tracker pipe
+        # - the Connection pipe
+        # - additionally, on posix + Python 3.8: multiprocessing's
+        #   resource_tracker pipe
+        if sys.version_info >= (3, 8) and os.name == 'posix':
+            n_expected_pipes = 4
+        else:
+            n_expected_pipes = 3
+        msg = ("Some pipes were not properly closed during the child process "
+               "setup.")
+        assert n_pipe == n_expected_pipes, msg
 
         # assert that the writable part of the Pipe (not passed to child),
         # have been properly closed.
@@ -514,6 +560,11 @@ class TestLokyBackend:
 
         return named_sem
 
+    @pytest.mark.skipif(
+        platform.python_implementation() == "PyPy" and
+        sys.version_info[:3] <= (3, 5, 3),
+        reason="early PyPy versions leak a file descriptor, see "
+               "https://bitbucket.org/pypy/pypy/issues/3021")
     def test_sync_object_handling(self):
         """Check the correct handling of semaphores and pipes with loky
 
@@ -569,14 +620,13 @@ class TestLokyBackend:
 
                 assert p.exitcode == 0
 
-    @pytest.mark.skipif(parallel_sum is None,
-                        reason="cython is not installed on this system.")
+    @with_parallel_sum
     def test_compatibility_openmp(self):
         # Use openMP before launching subprocesses. With fork backend, some fds
         # are nto correctly clean up, causing a freeze. No freeze should be
         # detected with loky.
-        parallel_sum(10)
-        p = self.Process(target=parallel_sum, args=(10,))
+        _run_openmp_parallel_sum(10)
+        p = self.Process(target=_run_openmp_parallel_sum, args=(100,))
         p.start()
         p.join()
         assert p.exitcode == 0
