@@ -7,17 +7,23 @@ import pytest
 import warnings
 import threading
 from time import sleep
-from tempfile import mkstemp
 from multiprocessing import util, current_process
 from pickle import PicklingError, UnpicklingError
+from distutils.version import LooseVersion
 
+import loky
 from loky import cpu_count
 from loky import get_reusable_executor
 from loky.process_executor import _RemoteTraceback, TerminatedWorkerError
 from loky.process_executor import BrokenProcessPool, ShutdownExecutorError
+from loky.reusable_executor import _ReusablePoolExecutor
+import cloudpickle
 
 from ._executor_mixin import ReusableExecutorMixin
-from .utils import TimingWrapper, id_sleep, check_subprocess_call, filter_match
+from .utils import TimingWrapper, id_sleep, check_python_subprocess_call
+from .utils import filter_match
+
+cloudpickle_version = LooseVersion(cloudpickle.__version__)
 
 # Compat windows
 if sys.platform == "win32":
@@ -26,7 +32,7 @@ if sys.platform == "win32":
 else:
     from signal import SIGKILL
     from ctypes.util import find_library
-    libc = ctypes.CDLL(find_library("libc"))
+    libc = ctypes.CDLL(find_library("c"))
 
 
 try:
@@ -43,9 +49,6 @@ except ImportError:
     pass
 
 
-INITIALIZER_STATUS = 'uninitialized'
-
-
 def clean_warning_registry():
     """Safe way to reset warnings."""
     warnings.resetwarnings()
@@ -81,9 +84,8 @@ def c_exit(exitcode=0):
     libc.exit(exitcode)
 
 
-def check_pids_exist_then_sleep(arg):
-    """Sleep for some time before returning
-    and check if all the passed pid exist"""
+def sleep_then_check_pids_exist(arg):
+    """Sleep for some time and the check if all the passed pids exist"""
     time, pids = arg
     sleep(time)
     res = True
@@ -222,7 +224,8 @@ class TestExecutorDeadLock(ReusableExecutorMixin):
         # on workers
         (return_instance, (CrashAtPickle,), TerminatedWorkerError, r"SIGSEGV"),
         (return_instance, (ExitAtPickle,), SystemExit, None),
-        (return_instance, (CExitAtPickle,), TerminatedWorkerError, r"EXIT\(0\)"),
+        (return_instance, (CExitAtPickle,), TerminatedWorkerError,
+         r"EXIT\(0\)"),
         (return_instance, (ErrorAtPickle,), PicklingError, None),
         # Check problem occuring while unpickling a task in
         # the result_handler thread
@@ -342,6 +345,16 @@ class TestExecutorDeadLock(ReusableExecutorMixin):
     @pytest.mark.parametrize("n_proc", [1, 2, 5, 13])
     def test_crash_races(self, n_proc):
         """Test the race conditions in reusable_executor crash handling"""
+
+        if (sys.platform == 'win32' and sys.version_info >= (3, 8)
+                and n_proc > 5):
+            pytest.skip(
+                "On win32, the paging size can be too small to import numpy "
+                "multiple times in the sub-processes (imported when loading "
+                "this file). Skipping while no better solution is found. See "
+                "https://github.com/joblib/loky/issues/279 for more details."
+            )
+
         # Test for external crash signal comming from neighbor
         # with various race setup
         executor = get_reusable_executor(max_workers=n_proc, timeout=None)
@@ -349,7 +362,7 @@ class TestExecutorDeadLock(ReusableExecutorMixin):
         pids = list(executor._processes.keys())
         assert len(pids) == n_proc
         assert None not in pids
-        res = executor.map(check_pids_exist_then_sleep,
+        res = executor.map(sleep_then_check_pids_exist,
                            [(.0001 * (j // 2), pids)
                             for j in range(2 * n_proc)])
         assert all(list(res))
@@ -483,7 +496,7 @@ class TestResizeExecutor(ReusableExecutorMixin):
         # the old one as it is still in a good shape. The resize should not
         # occur while there are on going works.
         pids = list(executor._processes.keys())
-        res1 = executor.submit(check_pids_exist_then_sleep, (.3, pids))
+        res1 = executor.submit(sleep_then_check_pids_exist, (.3, pids))
         clean_warning_registry()
         with warnings.catch_warnings(record=True) as w:
             # Cause all warnings to always be triggered.
@@ -548,7 +561,6 @@ class TestResizeExecutor(ReusableExecutorMixin):
         assert executor.submit(id_sleep, 42, 0.).result() == 42
         executor.shutdown()
 
-    @pytest.mark.timeout(30 if sys.platform == "win32" else 15)
     def test_resize_after_timeout(self):
         with warnings.catch_warnings(record=True) as recorded_warnings:
             warnings.simplefilter("always")
@@ -598,7 +610,7 @@ class TestGetReusableExecutor(ReusableExecutorMixin):
         with pytest.raises(ValueError):
             get_reusable_executor(max_workers=2, context='bad_start_method')
 
-    def test_interactively_define_executor_no_main(self):
+    def test_interactively_defined_executor_no_main(self):
         # check that the init_main_module parameter works properly
         # when using -c option, we don't need the safeguard if __name__ ..
         # and thus test LokyProcess without the extra argument. For running
@@ -606,19 +618,94 @@ class TestGetReusableExecutor(ReusableExecutorMixin):
         code = """if True:
             from loky import get_reusable_executor
             e = get_reusable_executor()
+
             e.submit(id, 42).result()
             print("ok")
         """
-        cmd = [sys.executable]
-        try:
-            fid, filename = mkstemp(suffix="_joblib.py")
-            os.close(fid)
-            with open(filename, mode='wb') as f:
-                f.write(code.encode('ascii'))
-            cmd += [filename]
-            check_subprocess_call(cmd, stdout_regex=r'ok', timeout=10)
-        finally:
-            os.unlink(filename)
+        check_python_subprocess_call(code, stdout_regex=r"ok")
+
+    def test_reused_flag(self):
+        executor, _ = _ReusablePoolExecutor.get_reusable_executor(
+            max_workers=2
+        )
+        executor, reused = _ReusablePoolExecutor.get_reusable_executor(
+            max_workers=2
+        )
+        assert reused
+        executor.shutdown(kill_workers=True)
+        executor, reused = _ReusablePoolExecutor.get_reusable_executor(
+            max_workers=2
+        )
+        assert not reused
+
+    @pytest.mark.xfail(cloudpickle_version >= LooseVersion("0.5.4") and
+                       cloudpickle_version <= LooseVersion("0.7.0"),
+                       reason="Known issue in cloudpickle")
+    # https://github.com/cloudpipe/cloudpickle/pull/240
+    def test_interactively_defined_nested_functions(self):
+        # Check that it's possible to call nested interactively defined
+        # functions and furthermore that changing the code interactively
+        # is taken into account by the single worker process.
+        code = """if True:
+            from loky import get_reusable_executor
+            e = get_reusable_executor(max_workers=1)
+
+            # Force a start of the children process:
+            e.submit(id, 42).result()
+
+            # Test that it's possible to call interactively defined, nested
+            # functions:
+
+            def inner_func(x):
+                return -x
+
+            def outer_func(x):
+                return inner_func(x)
+
+            assert e.submit(outer_func, 1).result() == outer_func(1) == -1
+
+            # Test that changes to the definition of the inner function are
+            # taken into account in subsequent calls to the outer function.
+
+            def inner_func(x):
+                return x
+
+            assert e.submit(outer_func, 1).result() == outer_func(1) == 1
+
+            print("ok")
+        """
+        check_python_subprocess_call(code, stdout_regex=r"ok")
+
+    def test_interactively_defined_recursive_functions(self):
+        # Check that it's possible to call a recursive function defined
+        # in a closure.
+        # Also check that calling several function that stems from the same
+        # factory with different closure states results in the expected result:
+        # the function definitions should not collapse in the single worker
+        # process.
+        code = """if True:
+            from loky import get_reusable_executor
+            e = get_reusable_executor(max_workers=1)
+
+            # Force a start of the children process:
+            e.submit(id, 42).result()
+
+            def make_func(seed):
+                def func(x):
+                    if x <= 0:
+                        return seed
+                    return func(x - 1) + 1
+                return func
+
+            func = make_func(0)
+            assert e.submit(func, 5).result() == func(5) == 5
+
+            func = make_func(1)
+            assert e.submit(func, 5).result() == func(5) == 6
+
+            print("ok")
+        """
+        check_python_subprocess_call(code, stdout_regex=r"ok")
 
     def test_compat_with_concurrent_futures_exception(self):
         # It should be possible to use a loky process pool executor as a dropin
@@ -713,14 +800,11 @@ class TestGetReusableExecutor(ReusableExecutorMixin):
 
 class TestExecutorInitializer(ReusableExecutorMixin):
     def _initializer(self, x):
-        global INITIALIZER_STATUS
-        INITIALIZER_STATUS = x
+        loky._initialized_state = x
 
     def _test_initializer(self, delay=0):
         sleep(delay)
-
-        global INITIALIZER_STATUS
-        return INITIALIZER_STATUS
+        return getattr(loky, "_initialized_state", "uninitialized")
 
     def test_reusable_initializer(self):
         executor = get_reusable_executor(
